package com.isu.gaswellwatch.vo.command.etc;

import com.isu.gaswellwatch.dto.modbus.ModbusCommandDto;
import com.isu.gaswellwatch.vo.command.Command;
import com.isu.gaswellwatch.vo.command.Timing;
import jakarta.validation.constraints.NotBlank;
import lombok.Getter;
import lombok.Setter;
import lombok.experimental.SuperBuilder;
import org.apache.commons.lang3.StringUtils;

import java.io.Serial;
import java.util.Collection;
import java.util.List;

/**
 * @author <a href="mailto:scwsl@foxmail.com">王仕龙</a>
 * 2024/11/27 23:04
 */
@Getter
@Setter
@SuperBuilder
public class SensorDelay extends Command implements Timing {

    @Serial
    private static final long serialVersionUID = -126267138442242492L;
    /**
     * 到达传感器延时时间。[0-999]:[0-59]:[0-59]
     */
    @NotBlank(message = "到达传感器延时时间不能为空")
    private String arrivalSensorDelayTime;
    /**
     * 关井时间。[0-999]:[0-59]:[0-59]
     */
    @NotBlank(message = "关井时间不能为空")
    private String wellShutInTime;
    /**
     * 续流时间。[0-999]:[0-59]:[0-59]
     */
    @NotBlank(message = "续流时间不能为空")
    private String afterFlowTime;

    public SensorDelay() {
        this.setCode("ETC.SENSOR_DELAY");
    }

    @Override
    public boolean validate() {
        return super.validate()
                && StringUtils.isNotBlank(this.afterFlowTime)
                && StringUtils.isNotBlank(this.wellShutInTime)
                && StringUtils.isNotBlank(this.arrivalSensorDelayTime);
    }

    @Override
    protected Collection<ModbusCommandDto> builderModbusCommand() {
        StringBuilder command = new StringBuilder(60);
        //                                0009*2->hex
        // 地址码  功能码  起始地址  连续长度  连续字长
        // 01     10     008B    0009     12
        command.append("0110008B000912")
                .append(this.toHexString(this.arrivalSensorDelayTime))
                .append(this.toHexString(this.wellShutInTime))
                .append(this.toHexString(this.afterFlowTime));
        return List.of(ModbusCommandDto.builder().command(command.toString()).length(16).build());
    }

    public void setAfterFlowTime(String afterFlowTime) {
        this.afterFlowTime = this.timingValidate(afterFlowTime, "afterFlowTime");
    }

    public void setWellShutInTime(String wellShutInTime) {
        this.wellShutInTime = this.timingValidate(wellShutInTime, "wellShutInTime");
    }

    public void setArrivalSensorDelayTime(String arrivalSensorDelayTime) {
        this.arrivalSensorDelayTime = this.timingValidate(arrivalSensorDelayTime, "arrivalSensorDelayTime");
    }

}
