package com.isu.gaswellwatch.vo.command.scss;

import com.isu.gaswellwatch.dto.modbus.ModbusCommandDto;
import com.isu.gaswellwatch.vo.command.Command;
import lombok.Getter;
import lombok.Setter;
import lombok.experimental.SuperBuilder;
import org.apache.commons.lang3.StringUtils;

import java.io.Serial;
import java.util.Collection;
import java.util.List;

/**
 * 控制定值数据寄存器地址
 *
 * @author <a href="mailto:scwsl@foxmail.com>王仕龙</a>
 * 2024/11/28 0:15
 */
@Getter
@Setter
@SuperBuilder
public class ControlMode extends Command {

    @Serial
    private static final long serialVersionUID = -3782424709053622334L;

    /**
     * 控制模式
     */
    private int ctlModel;
    /**
     * 套压最小值
     */
    private int minPressure;
    /**
     * 套压最大值
     */
    private int maxPressure;
    /**
     * 套压最小电压
     */
    private int pressureMinVoltage;
    /**
     * 套压最大电压
     */
    private int pressureMaxVoltage;
    /**
     * 油压最小值
     */
    private int oilMin;
    /**
     * 油压最大值
     */
    private int oliMax;
    /**
     * 油压最小电压
     */
    private int oilMinVoltage;
    /**
     * 油压最大电压
     */
    private int oilMaxVoltage;
    /**
     * 输压最小值
     */
    private int inputPressureMinValue;
    /**
     * 输压最大值
     */
    private int inputPressureMaxValue;
    /**
     * 输压最小电压
     */
    private int inputVoltageMinValue;
    /**
     * 输压最大电压
     */
    private int inputVoltageMaxValue;
    /**
     * 流量最小值
     */
    private int flowRateMinValue;
    /**
     * 流量最大值
     */
    private int flowRateMaxValue;
    /**
     * 流量最小电压
     */
    private int flowVoltageMinValue;
    /**
     * 流量最大电压
     */
    private int flowVoltageMaxValue;
    /**
     * 连续采样间隔
     */
    private int continuousSamplingIntervalDuration;
    /**
     * 到达传感器有效电平
     */
    private int sensorSignalEffectiveLevel;
    /**
     * 套压补偿极性
     */
    private int pressureCompensationPolarityFlag;
    /**
     * 套压补偿值
     */
    private int pressureCompensationValueSetting;
    /**
     * 油压补偿极性
     */
    private int oilPressureCompensationPolarityFlag;
    /**
     * 油压补偿值
     */
    private int oilPressureCompensationValueSetting;
    /**
     * 输压补偿极性
     */
    private int inputPressureCompensationPolarityFlag;
    /**
     * 输压补偿值
     */
    private int inputPressureCompensationValueSetting;
    /**
     * 流量补偿极性
     */
    private int flowCompensationPolarityFlag;
    /**
     * 流量补偿值
     */
    private int flowCompensationValueSetting;

    public ControlMode() {
        this.setCode("SCSS.CONTROL_MODE");
    }

    @Override
    protected Collection<ModbusCommandDto> builderModbusCommand() {
        StringBuilder command = new StringBuilder(250);
        // 地址码  功能码  起始地址  连续长度  连续字长
        // 01     10     0032    0036     6C
        command.append("0110003200366C");
        command.append(StringUtils.leftPad(Integer.toHexString(this.ctlModel), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.minPressure), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.maxPressure), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.pressureMinVoltage), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.pressureMaxVoltage), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.oilMin), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.oliMax), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.oilMinVoltage), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.oilMaxVoltage), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.inputPressureMinValue), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.inputPressureMaxValue), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.inputVoltageMinValue), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.inputVoltageMaxValue), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.flowRateMinValue), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.flowRateMaxValue), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.flowVoltageMinValue), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.flowVoltageMaxValue), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.continuousSamplingIntervalDuration), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.sensorSignalEffectiveLevel), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.pressureCompensationPolarityFlag), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.pressureCompensationValueSetting), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.oilPressureCompensationPolarityFlag), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.oilPressureCompensationValueSetting), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.inputPressureCompensationPolarityFlag), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.inputPressureCompensationValueSetting), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.flowCompensationPolarityFlag), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.flowCompensationValueSetting), 8, "0"));
        return List.of(ModbusCommandDto.builder().command(command.toString()).length(16).build());
    }
}
